Package: ChIPpeakAnno
Type: Package
Title: Batch annotation of the peaks identified from either ChIP-seq,
        ChIP-chip experiments, or any experiments that result in large
        number of genomic interval data
Version: 3.47.0
Encoding: UTF-8
Author: 
        Lihua Julie Zhu,
        Jianhong Ou, 
        Jun Yu, 
        Kai Hu, 
        Haibo Liu, 
        Junhui Li, 
        Hervé Pagès, 
        Claude Gazin,
        Nathan Lawson, 
        Ryan Thompson, 
        Simon Lin, 
        David Lapointe,
        Michael Green
Maintainer: Jianhong Ou <jou@morgridge.org>,
 Lihua Julie Zhu <julie.zhu@umassmed.edu>,
 Kai Hu <kai.hu@umassmed.edu>,
 Junhui Li <junhui.li@umassmed.edu>
Depends: R (>= 3.5), methods, IRanges (>= 2.13.12), GenomicRanges (>=
        1.31.8), S4Vectors (>= 0.17.25)
Imports: AnnotationDbi, BiocGenerics (>= 0.1.0), Biostrings (>=
        2.47.6), pwalign, DBI, dplyr, GenomeInfoDb, GenomicAlignments,
        GenomicFeatures, RBGL, Rsamtools, SummarizedExperiment,
        VennDiagram, biomaRt, ggplot2, grDevices, graph, graphics,
        grid, InteractionSet, KEGGREST, matrixStats, multtest,
        regioneR, rtracklayer, stats, utils, universalmotif, stringr,
        tibble, tidyr, data.table, scales, ensembldb
Suggests: AnnotationHub, BSgenome, limma, reactome.db, BiocManager,
        BiocStyle, BSgenome.Ecoli.NCBI.20080805,
        BSgenome.Hsapiens.UCSC.hg19, org.Ce.eg.db, org.Hs.eg.db,
        BSgenome.Celegans.UCSC.ce10, BSgenome.Drerio.UCSC.danRer7,
        BSgenome.Hsapiens.UCSC.hg38, DelayedArray, idr, seqinr,
        EnsDb.Hsapiens.v75, EnsDb.Hsapiens.v79, EnsDb.Hsapiens.v86,
        TxDb.Hsapiens.UCSC.hg18.knownGene,
        TxDb.Hsapiens.UCSC.hg19.knownGene,
        TxDb.Hsapiens.UCSC.hg38.knownGene, GO.db, gplots, UpSetR,
        knitr, rmarkdown, reshape2, testthat, trackViewer, motifStack,
        OrganismDbi, BiocFileCache
Description: 
        The package encompasses a range of functions for identifying the 
        closest gene, exon, miRNA, or custom features—such as highly conserved
        elements and user-supplied transcription factor binding sites. 
        Additionally, users can retrieve sequences around the peaks and obtain 
        enriched Gene Ontology (GO) or Pathway terms. In version 2.0.5 and beyond,
        new functionalities have been introduced. These include features for 
        identifying peaks associated with bi-directional promoters along with 
        summary statistics (peaksNearBDP), summarizing motif occurrences in 
        peaks (summarizePatternInPeaks), and associating additional identifiers 
        with annotated peaks or enrichedGO (addGeneIDs). The package integrates 
        with various other packages such as biomaRt, IRanges, Biostrings, BSgenome, 
        GO.db, multtest, and stat to enhance its analytical capabilities.
License: GPL (>= 2)
LazyLoad: yes
LazyData: true
LazyDataCompression: xz
biocViews: Annotation, ChIPSeq, ChIPchip
VignetteBuilder: knitr
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/ChIPpeakAnno
git_branch: devel
git_last_commit: 9964693
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:04:56 UTC; biocbuild
