# ChIPpeakAnno

[![platforms](http://bioconductor.org/shields/availability/devel/ChIPpeakAnno.svg)](http://bioconductor.org/packages/devel/bioc/html/ChIPpeakAnno.html)
[![build](http://bioconductor.org/shields/build/devel/bioc/ChIPpeakAnno.svg)](http://bioconductor.org/packages/devel/bioc/html/ChIPpeakAnno.html)

Batch annotation and visualization of peaks from ChIP-seq, ATAC-seq, and NAD-seq experiments or any experiments resulted in large number of chromosome ranges

## Installation

To install this package, start R and enter:

```r
library(BiocManager)
BiocManager::install("ChIPpeakAnno")
```

## Documentation

To view documentation of ChIPpeakAnno, start R and enter:
```r
browseVignettes("ChIPpeakAnno")
```
