require("ChIPpeakAnno") || stop("unable to load Package:ChIPpeakAnno")
require("BSgenome.Drerio.UCSC.danRer7") || stop("unable to load Package:BSgenome.Drerio.UCSC.danRer7")
require("BSgenome.Hsapiens.UCSC.hg19") || stop("unable to load Package:BSgenome.Hsapiens.UCSC.hg19")
require("org.Hs.eg.db") || stop("unable to load Package:org.Hs.eg.db")
require("org.Ce.eg.db") || stop("unable to load Package:org.Ce.eg.db")
require("BSgenome.Celegans.UCSC.ce10") || stop("unable to load Package:BSgenome.Celegans.UCSC.ce10")
require("EnsDb.Hsapiens.v79") || stop("unable to load Package:EnsDb.Hsapiens.v79")
require("TxDb.Hsapiens.UCSC.hg38.knownGene") || stop("unable to load TxDb.Hsapiens.UCSC.hg38.knownGene")
require("biomaRt") || stop("unable to load biomaRt")
require("reactome.db") || stop("unable to load reactome.db")
require("testthat") || stop("unable to load testthat")
test_check("ChIPpeakAnno")