#' An example GRanges object representing a ChIP-seq peak dataset
#' 
#' @name peaks2
#' @docType data
#' @format GRanges
#' @keywords datasets
#' @examples
#' 
#' data(peaks2)
#' head(peaks2, n = 2)
"peaks2"
