#' An example GRanges object representing a ChIP-seq peak dataset
#' 
#' @name peaks3
#' @docType data
#' @format GRanges
#' @keywords datasets
#' @examples
#' 
#' data(peaks3)
#' head(peaks3, n = 2)
"peaks3"
